Files
RedBear-OS/local/recipes/dev/libclc/source/clang/test/SemaCUDA/lambda.cu
T
vasilito cb424d7448 build: static patch-sanity linter (shift-left the malformed-patch class)
verify-patch-sanity.py validates every active recipe .patch has internally-
consistent hunk line counts — catching the 'malformed patch at line N' failure
at commit/CI/preflight time instead of hours into a cook. This cycle hit that
class three times (qtwaylandscanner, sddm, xwayland), each only discovered when
cookbook tried to apply the patch.

Running it across the repo found 29 latent malformed patches (validated against
GNU patch: e.g. relibc/P3-sysv-ipc reproduces 'malformed patch at line 22').
They were harmless only because they sit in vendored recipes (baked, not re-
applied) — but would fail on any version-bump re-derivation. --fix recounts the
hunk headers (body untouched) and repaired all 29.

Wired into build-preflight.sh (Phase 1.0D) and redbear-ci.yml, with a unit test
(test-patch-sanity.sh). Skips archived/legacy trees and unvalidatable formats
(empty placeholders, bare-@@ git hunks).
2026-08-01 05:13:02 +03:00

83 lines
2.9 KiB
Plaintext

// RUN: %clang_cc1 -std=c++17 -fsyntax-only -verify=com %s
// RUN: %clang_cc1 -std=c++17 -fsyntax-only -fcuda-is-device -verify=com,dev,warn %s
// RUN: %clang_cc1 -std=c++17 -fsyntax-only -fcuda-is-device -verify=com,dev,warn \
// RUN: -triple amdgcn-amd-amdhsa -target-cpu gfx906 -x hip %s
// RUN: %clang_cc1 -std=c++17 -fsyntax-only -fcuda-is-device -verify=com,dev \
// RUN: -Wno-gpu-maybe-wrong-side %s
#include "Inputs/cuda.h"
auto global_lambda = [] () { return 123; };
template<class F>
__global__ void kernel(F f) { f(); }
// dev-note@-1 3{{called by 'kernel<(lambda}}
// warn-note@-2 5{{called by 'kernel<(lambda}}
__host__ __device__ void hd(int x);
class A {
int b;
public:
void test() {
[=](){ hd(b); }();
[&](){ hd(b); }();
kernel<<<1,1>>>([](){ hd(0); });
kernel<<<1,1>>>([=](){ hd(b); });
// warn-warning@-1 {{capture host side class data member by this pointer in device or host device lambda function may result in invalid memory access if this pointer is not accessible on device side}}
kernel<<<1,1>>>([&](){ hd(b); });
// warn-warning@-1 {{capture host side class data member by this pointer in device or host device lambda function may result in invalid memory access if this pointer is not accessible on device side}}
kernel<<<1,1>>>([&] __device__ (){ hd(b); });
// warn-warning@-1 {{capture host side class data member by this pointer in device or host device lambda function may result in invalid memory access if this pointer is not accessible on device side}}
kernel<<<1,1>>>([&](){
auto f = [&]{ hd(b); };
// warn-warning@-1 {{capture host side class data member by this pointer in device or host device lambda function may result in invalid memory access if this pointer is not accessible on device side}}
f();
});
auto lambda1 = [this] __device__ { hd(this->b); };
// warn-warning@-1 {{capture host side class data member by this pointer in device or host device lambda function may result in invalid memory access if this pointer is not accessible on device side}}
kernel<<<1,1>>>(lambda1);
}
};
int main(void) {
auto lambda_kernel = [&]__global__(){};
// com-error@-1 {{kernel function 'operator()' must be a free function or static member function}}
int b;
[&](){ hd(b); }();
[=, &b](){ hd(b); }();
kernel<<<1,1>>>(global_lambda);
kernel<<<1,1>>>([](){ hd(0); });
kernel<<<1,1>>>([=](){ hd(b); });
kernel<<<1,1>>>([b](){ hd(b); });
kernel<<<1,1>>>([&](){ hd(b); });
// dev-error@-1 {{capture host variable 'b' by reference in device or host device lambda function}}
kernel<<<1,1>>>([=, &b](){ hd(b); });
// dev-error@-1 {{capture host variable 'b' by reference in device or host device lambda function}}
kernel<<<1,1>>>([&, b](){ hd(b); });
kernel<<<1,1>>>([&](){
auto f = [&]{ hd(b); };
// dev-error@-1 {{capture host variable 'b' by reference in device or host device lambda function}}
f();
});
return 0;
}